
# q-expansion of newform 8960.2.a.cc, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8960
weight = 2
poly_data = [-2, -6, 13, 8, -8, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[-1, 2, 0, 0, 0, 0], 1], [[0, 2, -5, -2, 1, 0], 2], [[6, 6, -7, -2, 1, 0], 2], [[-8, 18, 10, -9, -2, 1], 2], [[-2, 8, 6, -7, -2, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0, 0, 0], [0, 0, 0, -1, 0, 0], [-1, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0], [-1, 0, -1, -1, 0, 0], [-1, -1, 0, 1, 1, 0], [1, -1, 0, -1, 0, 0], [0, 1, -1, 0, -1, -1], [0, -1, -1, 0, -1, 0], [-3, 0, -1, 1, 1, -1], [1, 0, 1, 0, 1, 1], [0, -1, -1, 2, -1, 1], [0, -1, 1, 0, -1, 0], [-1, 2, 1, -2, 1, -1], [2, 0, -2, 1, 1, 1], [-2, 0, 2, -2, 0, 1], [-2, -2, -2, 0, 2, 1], [3, 0, 1, 2, 1, -1], [0, 0, 0, 0, 2, -2], [4, 0, 0, -4, 0, -1], [0, 2, 0, 0, -2, -1], [2, 1, 1, 1, -1, 1], [6, 0, 0, 0, 2, 2], [-2, -1, -1, 0, -1, 0], [3, 1, 0, 1, 0, -2], [-7, 2, 1, 0, -1, -1], [2, 0, -2, 1, 1, 1], [-6, 0, 2, -2, 0, 0], [1, -2, 1, 1, -1, 1], [7, 2, 3, -4, -1, -1], [0, 2, 2, -4, 0, -2], [-2, 1, 1, -4, -1, 1], [3, 0, 1, 2, 1, -1], [6, 2, 2, -2, -2, -1], [-2, 0, -2, 0, -2, 0], [-2, -3, 1, -1, 3, 1], [-4, -2, -4, 4, 0, 2], [-2, 0, 2, -2, -2, -2], [0, -2, -2, 3, -1, -1], [3, 1, -2, 3, 1, -2], [6, -2, 0, 0, 0, -2], [0, 0, 2, -6, 2, 0], [-8, -3, -5, 1, 1, 3], [3, 4, 1, 2, 1, -1], [6, -1, -3, 0, 1, 3], [6, 2, 4, -4, -2, 0], [1, 2, -1, 1, -2, 2], [0, -4, -4, 5, 3, 1], [0, -2, -2, 1, 2, 0], [-5, -2, 3, 2, 1, -1], [4, 2, 4, -2, 2, -2], [-6, -1, 3, 1, 1, -1], [2, 3, 3, -6, 1, -2], [6, -2, -2, -2, -2, -3], [6, 0, 4, 0, -2, -1], [-8, 2, -2, 6, -2, -2], [-12, -2, 0, 4, -2, 0], [5, -2, -1, 0, 1, -1], [4, -4, 0, 0, -2, -1], [-2, -1, -1, -3, 0, -3], [10, 0, -2, 3, -2, 0], [7, 1, -2, -3, -1, 0], [4, 2, 2, 3, 2, -2], [7, 6, 1, -6, -3, -3], [19, -1, 2, 3, 2, 0], [4, -2, -2, 4, 2, 1], [6, -2, -4, 4, 4, 0], [8, 0, -2, 0, 0, -4], [-1, 2, 1, -2, -1, 5], [-9, -2, -5, 4, 5, 3], [7, -1, -2, -3, -2, 0], [-12, 0, 4, 0, 4, -1], [-4, -2, -2, -1, 3, -3], [6, -1, 1, 8, 3, -3], [-4, -4, 0, 4, 0, -2], [6, 0, 2, 0, -2, 0], [-9, 0, 1, 1, 3, 3], [5, -3, 4, 1, 1, 4], [4, 1, -1, -1, 2, 5], [16, 0, -2, 0, 0, 4], [-10, 4, 0, 0, 0, -1], [-1, -4, -3, -5, -1, -3], [0, -1, 1, -3, 3, -3], [2, -2, -6, -4, 0, -1], [2, -2, -4, 6, -4, 4], [-5, -6, -3, -2, 1, 3], [0, -1, -3, 7, 0, -3], [9, 4, -5, 2, -3, -1], [0, 2, 0, -6, -2, -2], [-4, 1, 1, 6, 3, -2], [12, -6, -6, 7, -2, 2], [13, 0, 5, 2, 7, 1], [0, 4, 0, -6, -2, 2], [9, 2, -1, 3, 0, -4], [-5, 0, -1, -1, 0, -2], [-2, 2, 0, 7, -3, 1], [-14, 6, 2, -4, -4, 0], [12, 1, -1, -2, -1, 0], [2, -2, 2, 4, -4, 2], [19, 0, 5, -1, -1, 3], [-17, -2, 1, 4, -1, 1], [6, 0, -2, -2, -2, 1], [8, 4, 2, 0, 0, 0], [6, -4, 4, 4, 4, 4], [8, 2, 2, 8, 2, 2], [5, -3, 2, 9, 2, -4], [-4, -2, 0, 8, 2, 0], [-1, -1, 2, -1, 2, 2], [-6, 1, 1, -7, -1, 5], [16, 4, 6, 0, -4, -2], [26, 2, 0, -7, -1, -3], [0, 6, -2, 0, -2, -5], [-4, 4, -2, -4, -8, 0], [22, -1, -1, 0, -3, -1], [6, 3, -1, 1, 1, 1], [6, 0, 0, 4, -4, 8], [-18, -2, 0, 5, 0, 0], [-8, -2, -2, -4, 6, -4], [8, -1, 3, -2, -7, 1], [11, 0, 3, 1, 2, -6], [8, 6, 0, 6, -2, -6], [-7, -4, 3, 6, -1, -3], [-1, 1, 2, -11, 3, 0], [7, 6, 1, 4, -5, 1], [-4, 0, 2, 4, 4, -3], [7, -2, -1, 5, -3, 1], [-15, 0, -1, 3, -7, -5], [21, 0, -3, -2, 3, 5], [-2, -4, 2, -4, 6, -2], [13, 1, 4, 3, 3, 4], [-9, -2, -3, 13, 0, -2], [4, 2, -2, 8, -4, -2], [-6, -1, 7, 3, 5, -3], [14, 2, 4, -2, 4, 1], [2, -1, -1, 2, -3, -4], [12, 2, 4, 0, -6, -2], [7, -1, 4, 13, -3, 2], [10, 6, 4, -5, -8, 0], [-13, 1, -2, 1, -1, 6], [-18, -1, -1, -1, 2, -1], [22, -2, 2, 6, 2, -3], [-11, -8, -5, 5, 5, 7], [-8, 1, 9, 2, 5, 2], [2, 0, -6, 6, -2, 2], [-15, -2, -3, -2, -9, 1], [-15, 4, 5, -2, 3, 1], [8, 2, 4, 0, 4, 0], [-14, 6, 2, 4, -4, -1], [-24, 0, 2, -4, -4, -1], [-20, 1, -3, 6, 5, 2], [-4, -3, -3, 6, -5, -3], [10, 4, 12, 2, -2, -4], [3, 2, 9, 4, 7, 5], [10, 6, -4, 4, -4, -4], [-1, 2, 5, 6, 7, 1], [6, 0, -2, -4, 2, 7], [-12, -3, -9, 9, 9, -1], [-8, 0, -2, 2, -2, -2], [15, -5, -2, -1, 2, 0], [-3, 0, -1, 6, 3, -1], [9, 2, 7, -4, -1, -3], [-7, 4, 15, -6, 1, -3], [0, -2, -10, 2, 2, 6], [-10, -5, -1, 2, -3, 1], [5, 2, 5, 0, -1, 3], [22, 6, 4, 5, -1, -9], [-8, -8, 4, 0, 0, 1], [7, 1, -2, -3, -1, 0], [2, 1, 5, -1, 8, -1], [0, 0, -6, 0, -2, 2], [0, 0, 4, 0, 4, 8], [12, -4, 2, -4, -2, 2], [-16, -9, -3, 5, -1, 3], [-6, 0, -4, 4, 6, -4], [-1, -2, -7, 2, 1, -3], [-22, 8, 8, 2, 2, 4], [-15, -4, 1, -1, 2, -4], [-13, 2, 3, -5, -3, 3], [-12, -2, -2, 0, -2, 2], [-1, -8, 1, 10, 1, 5], [-20, 3, -9, 2, 1, -6], [-8, 1, -1, 6, -5, 1], [24, 1, -3, -2, 3, -1], [-10, -2, 2, -8, -4, 5], [12, 7, 3, 2, -3, -4], [-11, -4, 7, 4, 7, 5], [2, 4, 2, -14, -4, -1], [6, 6, -4, -3, 0, 2], [16, 1, 3, 7, -2, -3], [3, -2, -3, 4, -5, 7], [12, 4, 6, 4, 0, 1], [-21, -6, -3, -2, -3, 3], [5, 4, 5, -5, -2, -4], [-11, -2, -7, 14, 3, -5], [8, 4, 8, -4, -4, -4], [1, -7, -6, 5, 6, 4], [-4, 5, 3, -11, 4, -5], [14, 7, -7, -4, -7, 1], [-19, -1, -8, -1, -4, 2], [12, -6, 2, 3, 3, 1], [-12, -8, -12, -4, 0, 4], [26, 4, 6, 6, 0, -2], [-18, 2, 6, 4, -2, 2], [24, 5, -5, 2, -1, 2], [16, 3, -7, 12, 5, -1], [5, 1, 4, 9, -3, 2], [4, 6, 6, -2, 10, -4], [-3, -6, -1, 8, 5, 7], [-30, -4, 0, 8, 0, 0], [-8, 5, -5, -6, -5, 1], [13, 0, 3, 2, -3, 1], [-4, 2, 0, -8, 6, 0], [8, -7, -3, 14, -3, 4], [6, -8, 2, -1, -6, 4], [2, -4, -2, 6, 4, 2], [2, -7, -7, 4, -3, 4], [-4, 5, -3, -12, -3, 4], [-20, -5, -7, 8, -1, 2], [-4, 5, 1, -12, -5, 0], [36, 2, 2, 0, -4, -1], [30, 0, -6, 0, 2, 2], [-2, 4, 2, -2, 4, 6], [-26, -7, 1, 7, 8, -1], [-16, -6, 2, -6, 2, 8], [36, 2, -2, -3, 2, 0], [2, 0, -2, -12, 6, -4], [-23, 1, -6, 9, -6, -6], [22, -1, 3, 3, -3, 1], [-2, 8, -2, -15, 1, 3], [8, 2, 10, 8, 14, 2], [2, 3, -3, -16, -9, -1], [4, 0, -6, -2, -12, 5], [-10, -12, -2, 12, 2, 7], [-14, -9, 7, -7, 4, 5], [2, -8, -2, -5, 2, 4], [-4, 8, 4, -15, -5, 1], [-8, -1, 1, 20, 3, 2], [2, 4, -2, 6, 6, -9], [7, -2, 9, -5, 2, 6], [-11, 8, -3, 2, -1, -3], [0, 0, -4, 0, 0, 8], [8, 4, 2, 4, 4, 5], [-2, 10, -4, 7, 5, -1], [0, 8, 0, 8, -4, 4], [-1, -9, -10, 9, -2, 10], [19, 2, 9, 2, -3, -5], [-6, -4, 2, -8, -6, 8], [-16, 2, 4, -14, -2, -1], [17, -4, -3, -3, 4, -4], [-24, -1, -1, 0, -11, -4], [4, 1, -11, 14, 3, -3], [-12, -6, -8, 16, 10, 0], [20, -2, 6, 4, 0, 0], [-6, -10, -2, -6, -8, -2], [38, -6, 6, 0, -2, 6], [31, 6, 1, -15, -4, -4], [30, 0, -6, -4, -2, -4], [-24, 0, 6, 0, -4, -4], [9, -3, 4, -11, 9, -4], [-7, -5, -4, -9, 4, -6], [-12, -1, -1, -4, -3, -4], [-10, -4, 2, 2, 2, 10], [-18, -4, -12, 18, 0, 0], [23, 5, 10, -3, -5, -2], [-18, 8, 8, -12, -6, 3], [-10, 11, 15, -4, -3, -5], [-17, 4, 1, 13, -9, 1], [-20, -6, 0, 4, -2, 0], [-20, 0, -8, 0, -6, -6], [-8, -5, 7, -6, 1, -3], [15, 6, 7, 2, -3, -1], [23, 6, 13, -8, 1, -1], [11, 7, 2, 17, -2, 4], [-28, -4, -8, 16, -8, 3], [-14, -4, 4, -8, 2, 7], [10, 6, 0, 17, -1, -5], [22, -8, -6, 16, 6, -6], [-12, 12, 8, -8, 0, -8], [4, 3, 5, -11, -6, -1], [40, -4, 6, -2, 8, 7], [18, 0, -6, 5, 1, -1], [0, -1, 5, 13, 7, -1], [-2, -2, -8, 12, 4, -8], [12, -2, 8, -4, -2, -2], [26, 0, -6, 1, -6, 2], [-18, 3, -1, -1, -11, -3], [5, -2, -7, 0, -13, 1], [24, 9, 9, 10, -9, -1], [22, 2, -4, 8, 2, 2], [-16, 2, 4, -14, 8, -6], [-11, -10, 5, 7, 11, 3], [-10, 0, 6, 1, 6, -2], [9, 8, 3, -6, 1, -7], [2, 9, 9, -10, 3, -1], [-32, 1, 1, -2, -5, -3], [5, 6, 1, 2, -1, 7], [4, 10, 2, 2, -6, -14], [-5, -1, -4, -3, 5, 4], [-4, 2, 0, -12, -6, -1], [2, 6, -8, 15, 4, -2], [21, 1, 10, -3, -6, 4], [16, -2, 2, -4, 8, -9], [0, -1, 5, 13, -5, -9], [-10, 0, 14, -12, 2, -2], [-37, -6, -3, -7, -4, 10], [-5, -2, -5, 0, -1, -1], [1, 6, 11, -6, 3, -11], [14, -2, 2, 6, 0, -2], [-20, -6, -2, -8, 2, 3], [22, 10, 12, -2, 2, -9], [-10, -8, -6, -4, 2, 4], [25, -4, 3, -1, 9, 11], [-14, 4, 8, -16, 0, -4], [-24, -4, -12, 2, 2, 6], [-24, -3, -3, -4, 3, 10], [-2, -7, 1, -16, 1, 0], [0, -6, -8, 10, 2, 3], [4, -5, -7, 11, -1, -7], [5, 11, 4, -5, 8, -2], [-36, -3, -5, 7, -10, 1], [-16, 10, -8, 6, -10, -13], [-13, -4, 1, 10, 7, -3], [-16, 6, -8, -12, -2, -4], [0, 7, -9, -2, 3, 4], [-30, 4, -4, 0, -12, -2], [12, -10, 0, 14, 0, 4], [-1, -6, -7, 11, -6, 8], [-10, 8, 8, -12, -6, 2], [-4, -7, 9, 2, 9, 0], [-22, -13, -7, 20, -3, 7], [-24, -4, -4, 0, 0, 0], [14, 1, 11, -4, -1, 9], [16, -6, 6, -14, 6, 0], [2, -4, 2, 17, -2, -4], [8, 6, 2, 8, -2, -8], [-18, -6, -16, 17, -4, -2], [-15, -6, -7, 3, 11, 9], [-18, -6, -10, -4, 0, 5], [12, 7, 9, 2, -11, 2], [-14, -6, -12, 3, 1, 1], [40, 8, 8, 12, 2, -3], [-18, -2, -10, 18, -2, -4], [-9, 0, -11, -12, -7, 3], [-6, -1, -5, 15, 1, 3], [16, 11, 3, -10, -5, -3], [34, 10, 4, -12, -8, -12], [-4, -2, 0, -4, 10, 4], [41, 10, -5, -12, -11, -5], [1, 4, -3, 10, 11, 1], [-13, -5, -4, 1, -11, -4], [-18, 6, -6, 8, -6, -7], [23, 0, -3, 6, -3, 9], [9, 6, 13, 5, -1, 7], [18, -10, -4, -11, 3, 1], [-27, -4, -9, 1, 1, 3], [-8, -14, -8, 20, -2, 5], [28, 8, -4, -6, -10, -2], [-14, -13, -9, 8, -7, 11], [-21, -6, 3, -12, -7, 3], [8, 0, -4, 2, 2, 4], [-50, -7, -5, 0, -3, 9], [-8, 8, 18, -10, -2, -2], [-28, -3, -11, 14, 9, 2], [-21, 0, -9, 11, 12, -2], [-10, 2, 10, -12, 0, 4], [13, -10, -7, 4, 15, 5], [21, 1, -8, 5, 1, 8], [0, 1, 5, -4, -1, -8], [14, -13, 3, -1, 6, 15], [9, 0, 5, -15, -4, 6], [20, 5, 15, -6, -1, -5], [-36, -10, -10, 8, 8, 6], [17, 0, 3, -1, -7, -3], [-8, -8, 8, -10, -2, 4], [8, 0, 4, 12, 0, -7], [-12, -4, -10, 16, 12, -7], [-18, 11, 3, -23, 1, -1], [31, 13, 8, -13, -4, -10], [6, -12, -4, 0, 12, 4], [45, 7, 8, -19, -4, -14], [2, 0, -4, 8, 8, -4], [8, -8, -6, 12, 8, -10], [-4, 6, -2, -8, 0, -4], [-6, -12, -8, -4, 2, 6], [-22, -7, -11, 6, -1, -3], [4, 4, 8, -15, -5, -7], [26, 3, 11, -11, -15, 5], [21, -1, -6, 5, 3, 4], [20, 0, 4, 16, 0, 0], [-30, 12, -2, -15, -3, -1], [-2, 5, -7, 21, -10, 1], [43, -1, 0, 5, 1, 12], [-19, -4, -3, -1, 2, 0], [2, 2, 10, -4, -8, 2], [-16, -13, -15, 7, 3, 7], [-2, -6, 2, -2, -4, 16], [9, 2, -9, 8, 5, 1], [-28, 3, -15, -12, 3, 0], [39, 2, 5, 9, 12, -4], [4, 8, 8, -20, -12, 0], [1, 4, 3, 2, -9, 1], [-18, -2, -6, -6, -14, -4], [-48, -7, -3, 4, 1, 4], [15, 1, -4, 1, -8, 6], [-18, 6, 10, -6, -4, 8], [16, 0, -8, -10, 2, -4], [-32, 0, 0, -8, -2, 3], [14, -1, -5, -13, 6, 11], [24, 8, 6, -16, 0, -10], [4, -4, -6, 12, 8, 1], [11, -9, 6, 7, 2, 8], [0, 0, 8, 4, 4, 19], [-7, 10, 15, -2, 1, -1], [22, -6, 4, -4, -8, -6], [-17, -14, -9, 0, 3, 11], [-12, 14, 20, -14, 2, 0], [-4, -3, -9, 17, -3, 7], [34, 8, -2, -4, -2, -12], [-28, 2, 8, -8, 6, 1], [30, -4, 2, 9, 1, 7], [-7, -6, 1, -1, -9, 1], [4, 2, 4, -8, 8, 6], [-12, 4, -4, 18, -8, 0], [36, 8, 6, -16, -16, 1], [3, 9, 8, -11, 4, -8], [9, 3, -6, -21, -3, -2], [50, 4, 10, -8, -8, 4], [34, -2, 6, 8, -6, 8], [-50, 3, 3, 2, 1, 1], [-13, 2, 9, 0, 7, -9], [-10, 9, -3, 14, -13, -4], [6, -6, 8, 12, 4, -4], [-2, -4, 10, -4, -2, 4], [-28, -8, 8, 5, 7, 7], [-7, 15, -2, 5, -5, -4], [34, 10, -10, -14, -8, 4], [-18, 11, -1, -1, 6, 3], [-9, 6, 11, 3, -3, -3], [-9, -6, 5, 0, 1, -13]]
